﻿;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Write the error log to disk in the temporary directory
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure WriteErrorLog(ModuleName.s = #Empty$, Message.s = #Empty$)
  
  ;------------------------------------------------------------------------------------------------
  ; First time this has run for the current session so create a new log file, overwrite the old one
  ;------------------------------------------------------------------------------------------------
  
  If ListSize(ErrorList()) <> #NoErrorLogEntries  And Program\FirstTime = #False
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\FirstTime = #True
    
    ;----------------------------------------------------------------------------------------------
    ; Setup the formatted date value or the header
    ;----------------------------------------------------------------------------------------------
    
    DateVal = Date()
    
    CurrentDate.s = NameOfDay(DayOfWeek(DateVal))    + " "
    
    CurrentDate.s + Str(Day(DateVal))                + " "
    
    CurrentDate.s + NameOfMonth(Month(DateVal) - 1)  + " "
    
    CurrentDate.s + Str(Year(DateVal))
    
    ;----------------------------------------------------------------------------------------------
    ; Try to create the file now
    ;----------------------------------------------------------------------------------------------
    
    ErrorLogId.i = CreateFile(#PB_Any, Program\ErrorFile)
    
    ;----------------------------------------------------------------------------------------------
    ; If we got a file handle, write the header details to it
    ;----------------------------------------------------------------------------------------------
    
    If ErrorLogId.i
      
      WriteStringN(ErrorLogId.i, "----------------------------------------------------------------------")
      
      WriteStringN(ErrorLogId.i, "New program run for: " + #BaseName + #Version + Space(1) + CurrentDate.s)
      
      WriteStringN(ErrorLogId.i, "----------------------------------------------------------------------")
      
      CloseFile(ErrorLogId.i)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf

  ;------------------------------------------------------------------------------------------------
  ; Errors are ready to be written to the file if list is full or routine is called at program end
  ;------------------------------------------------------------------------------------------------

  If ListSize(ErrorList()) > 25
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the previously created ErrorList log file
    ;----------------------------------------------------------------------------------------------
    
    ErrorLogId.i = OpenFile(#PB_Any, Program\ErrorFile)
    
    ;----------------------------------------------------------------------------------------------
    ; Now let's see if we got a file handle
    ;----------------------------------------------------------------------------------------------
    
    If ErrorLogId.i
      
      ;--------------------------------------------------------------------------------------------
      ; Seek to the end of the file to add entries
      ;--------------------------------------------------------------------------------------------
      
      FileSeek(ErrorLogId.i, Lof(ErrorLogId.i))
      
      ;--------------------------------------------------------------------------------------------
      ; Iterate through the list of errors
      ;--------------------------------------------------------------------------------------------
      
      ForEach ErrorList()
        
        WriteString(ErrorLogId.i,   FormatField("Time: ",     18) + ErrorList()\ErrorTime     + "|")
        
        WriteString(ErrorLogId.i,   FormatField("Module: ",   18) + ErrorList()\ErrorModule   + "|")
        
        WriteStringN(ErrorLogId.i,  FormatField("Message: ",  18) + ErrorList()\ErrorMessage)
        
      Next
      
      ;--------------------------------------------------------------------------------------------
      ; Close the error log
      ;--------------------------------------------------------------------------------------------
      
      CloseFile(ErrorLogId.i)
      
      ;--------------------------------------------------------------------------------------------
      ; Clear the list to start again
      ;--------------------------------------------------------------------------------------------
      
      ClearList(ErrorList())
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    AddElement(ErrorList())
    
    ErrorList()\ErrorTime     = FormatDate("%hh:%ii:%ss", Date())
    
    ErrorList()\ErrorModule   = ModuleName.s
    
    ErrorList()\ErrorMessage  = Message.s
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Errors are ready to be written to the file if list is full or routine is called at program end
  ;------------------------------------------------------------------------------------------------
  
  If ModuleName.s = "Main module"
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the previously created ErrorList log file
    ;----------------------------------------------------------------------------------------------
    
    If ListSize(ErrorList()) > #NoItems
      
      ;--------------------------------------------------------------------------------------------
      ; Now let's see if we got a file handle
      ;--------------------------------------------------------------------------------------------
      
      ErrorLogId.i = OpenFile(#PB_Any, Program\ErrorFile)
      
      ;--------------------------------------------------------------------------------------------
      ; Now let's see if we got a file handle
      ;--------------------------------------------------------------------------------------------
      
      If ErrorLogId.i
        
        ;------------------------------------------------------------------------------------------
        ; Seek to the end of the file to add entries
        ;------------------------------------------------------------------------------------------
        
        FileSeek(ErrorLogId.i, Lof(ErrorLogId.i))
        
        ;------------------------------------------------------------------------------------------
        ; Iterate through the list of errors
        ;------------------------------------------------------------------------------------------
        
        ForEach ErrorList()
          
          WriteString(ErrorLogId.i,   FormatField("Time: ",     18) + ErrorList()\ErrorTime     + "|")
          
          WriteString(ErrorLogId.i,   FormatField("Module: ",   18) + ErrorList()\ErrorModule   + "|")
          
          WriteStringN(ErrorLogId.i,  FormatField("Message: ",  18) + ErrorList()\ErrorMessage)

        Next
        
        ;------------------------------------------------------------------------------------------
        ; Close the error log
        ;------------------------------------------------------------------------------------------
        
        CloseFile(ErrorLogId.i)
        
        ;------------------------------------------------------------------------------------------
        ; Clear the list to start again
        ;------------------------------------------------------------------------------------------
        
        ClearList(ErrorList())
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ;
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ;
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 213
; Folding = -
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0