;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Export appointment data to a standard XML file
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure ExportDatabase()

  ;------------------------------------------------------------------------------------------------
  ; Process the current table name and dump the contents to an XML file on disk
  ;------------------------------------------------------------------------------------------------
  
  QueryString.s = "SELECT * FROM MyInfo ORDER BY Title ASC"
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)  <> #DatabaseQueryFail
    
    ;----------------------------------------------------------------------------------------------
    ; Process the next returned row
    ;----------------------------------------------------------------------------------------------
    
    While NextDatabaseRow(Program\DatabaseHandle)
      
      ;--------------------------------------------------------------------------------------------
      ; Have we created this XML tree already?
      ;--------------------------------------------------------------------------------------------
      
      If *XmlTree = #False                                                                          ; We haven't created this xml before so do so now
        
        *XmlTree = CreateXML(#PB_Any)                                                               ; Creates a new empty XML tree identified by the #XML number
        
        *RootXmlNode = CreateXMLNode(RootXMLNode(*XmlTree), #Empty$)                                ; Creates a new XML node and inserts it into the given parent node. 

        SetXMLNodeName(*RootXmlNode, "MyInfoList")                                                  ; Changes the tagname of the given XML node (root in this case)
        
      EndIf                                                                                         ;  No more tests to run
      
      ;--------------------------------------------------------------------------------------------
      ; Creates a new XML node and inserts it into main XML node of the tree.
      ;--------------------------------------------------------------------------------------------
      
      *RowNode = CreateXMLNode(MainXMLNode(*XmlTree), "MyInfoData")
      
      ;--------------------------------------------------------------------------------------------
      ; If the main node was created, dump all the columns using the column names as node names
      ;--------------------------------------------------------------------------------------------
      
      If *RowNode
        
        ;------------------------------------------------------------------------------------------
        ; Get the column names and write them first
        ;------------------------------------------------------------------------------------------
        
        NumberOfColumns.i = DatabaseColumns(Program\DatabaseHandle)
        
        ;------------------------------------------------------------------------------------------
        ; Iterate through the columns
        ;------------------------------------------------------------------------------------------
        
        For ColumnNamesLoop.i = 0 To NumberOfColumns.i -1
          
          CurrentColumn.s = DatabaseColumnName(Program\DatabaseHandle, ColumnNamesLoop.i)
          
          *ColumnNode = CreateXMLNode(*RowNode, CurrentColumn.s)                                   ; Creates a new XML node and inserts it into main XML node of the tree.
            
          SetXMLNodeText(*ColumnNode, TextToXml(GetDatabaseString(Program\DatabaseHandle,  ColumnNamesLoop.i)))
          
        Next                                                                                        ; Next column
        
        ;------------------------------------------------------------------------------------------
        ; Main node wasn't created, exit out of this procedure
        ;------------------------------------------------------------------------------------------
        
      Else
        
        SetInfoBarArea("Headings",  "Error",  "Could not create the export data file.", #PB_Compiler_Procedure)
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Wend
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    FinishDatabaseQuery(Program\DatabaseHandle)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    If *XmlTree
      
      FormatXML(*XmlTree, #PB_XML_ReFormat)
      
      SaveXML(*XmlTree, Program\BackupDirectory + "MyInfo_Export.xml")
      
      FreeXML(*XmlTree)
      
      SetInfoBarArea("Headings",  "Info",  "Data exported to " + Program\BackupDirectory, #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; The database query has failed for some reason
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Error",  "The database query failed or was empty. " + DatabaseError(), #PB_Compiler_Procedure)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  TextFileId.i = CreateFile(#PB_Any, Program\BackupDirectory + "MyInfo_Export.txt")
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If TextFileId.i <> #NoExportFileHandle
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    ;QueryString.s = "SELECT * FROM MyInfo ORDER BY Title ASC"
    
    If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)  <> #DatabaseQueryFail
      
      While NextDatabaseRow(Program\DatabaseHandle)
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  0), 20) + KillQuote(GetDatabaseString(Program\DatabaseHandle,   0)))                   ; Title
        
        WriteStringN(TextFileId.i, #Empty$)                                                                                                                                       ; 
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  1), 20) + KillQuote(GetDatabaseString(Program\DatabaseHandle,   1)))                   ; Information
        
        WriteStringN(TextFileId.i, #Empty$)                                                                                                                                       ; 
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  2), 20) +  KillQuote(GetDatabaseString(Program\DatabaseHandle,  2)))                   ; Category
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  3), 20) +            GetDatabaseString(Program\DatabaseHandle,  3))                    ; Archived
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  4), 20) +            GetDatabaseString(Program\DatabaseHandle,  4))                    ; Favourite
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  5), 20) +            GetDatabaseString(Program\DatabaseHandle,  5))                    ; Locked
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  6), 20) +            GetDatabaseString(Program\DatabaseHandle,  6))                    ; Deleted
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  7), 20) +    IsoDate(GetDatabaseString(Program\DatabaseHandle,  7), "NormalDate"))     ; Updated

        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  8), 20) +  KillQuote(GetDatabaseString(Program\DatabaseHandle,  8)))                   ; Owner
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle,  9), 20) +            GetDatabaseString(Program\DatabaseHandle,  9))                    ; Attachments
        
        WriteStringN(TextFileId.i, FormatField(DatabaseColumnName(Program\DatabaseHandle, 10), 20) +            GetDatabaseString(Program\DatabaseHandle, 10))                    ; Recordid
        
        WriteStringN(TextFileId.i, "___________________::: End Of Record ::: __________________")
        
        WriteStringN(TextFileId.i, #Empty$)
        
      Wend
      
      FinishDatabaseQuery(Program\DatabaseHandle)
      
      MessageString.s = "MyInfo XML data and text files have been saved in this directory:" + #CRLF$ + #CRLF$
      
      MessageString.s + Program\BackupDirectory
      
      MessageRequesterEBS("Export finished", MessageString.s, #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
      
      ;MessageRequester("Export finished", MessageString.s)

    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFile(TextFileId)  
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------

EndProcedure


; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 150
; FirstLine = 135
; Folding = -
; EnableXP
; HideErrorLog
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant