;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Show a list of all the keyboard shortcuts
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure KeyboardShortcuts()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if a previous window is open
  ;------------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the data window
    ;----------------------------------------------------------------------------------------------
    
    If Window_Shortcuts()
      
      ;--------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_MyInfo, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Warn other components a window is open and not to attempt to open another
      ;--------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;--------------------------------------------------------------------------------------------
      ; Add the various keyboard shortcuts
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Shortcuts, #ShortcutValue_EscapeShortcuts, #KeyboardShortcut_EscapeShortcuts)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the common window and statusbar message
      ;--------------------------------------------------------------------------------------------
      
      Message.s = "Show " + #Program + " keyboard shortcuts"
      
      ;--------------------------------------------------------------------------------------------
      ; Use the common message as a window identifier
      ;--------------------------------------------------------------------------------------------
      
      Program\LastWindow = Message.s
      
      ;--------------------------------------------------------------------------------------------
      ; Set the window title with the common message and next record number
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Shortcuts, Message.s)
      
      ;--------------------------------------------------------------------------------------------
      ; Remove borders from the details pane by remove windows theming support, gets rid of 1 pixel border
      ;--------------------------------------------------------------------------------------------
      
      RemoveBorders(#Gadget_Shortcuts_Keylist)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      LineCounter.i = 0
      
      ;--------------------------------------------------------------------------------------------
      ; Setup some quickie display macros for headings to avoid repeated typing
      ;--------------------------------------------------------------------------------------------
      
      Macro AddHeading(FirstString, SecondString)
        
        AddGadgetItem(#Gadget_Shortcuts_Keylist, #AtTheEndOfTheList, #Empty$        + #LF$ + #Empty$)
        
        LineCounter.i + 1
        
        AddGadgetItem(#Gadget_Shortcuts_Keylist, #AtTheEndOfTheList, FirstString    + #LF$ + SecondString)
        
        SetGadgetItemColor(#Gadget_Shortcuts_Keylist, LineCounter.i, #PB_Gadget_BackColor,  $9D1206)
        
        SetGadgetItemColor(#Gadget_Shortcuts_Keylist, LineCounter.i, #PB_Gadget_FrontColor, $FFFFFF)
        
        LineCounter.i + 1
        
        AddGadgetItem(#Gadget_Shortcuts_Keylist, #AtTheEndOfTheList, #Empty$        + #LF$ + #Empty$)
        
        LineCounter.i + 1
        
      EndMacro
      
      ;--------------------------------------------------------------------------------------------
      ; Setup some qhuckie display macros for content to avoid repeated typing
      ;--------------------------------------------------------------------------------------------
      
      Macro AddEntry(Shortcut, Keytext, Imagenumber)
        
        AddGadgetItem(#Gadget_Shortcuts_Keylist, #AtTheEndOfTheList, GetShortCutText(Shortcut) + #LF$ + Keytext, ImageID(Imagenumber))
        
        LineCounter.i + 1
        
      EndMacro

      ;--------------------------------------------------------------------------------------------
      ; File menu, main window
      ;--------------------------------------------------------------------------------------------
      
      AddHeading(#Empty$, "File menu, main window")
      
      AddEntry(#ShortcutValue_BackupDatabase,           LongTooltip\BackupDatabase,                  #MenuBarIcon_MyInfo_BackupDatabase)
      
      AddEntry(#ShortcutValue_ExportDatabase,           LongTooltip\ExportDatabase,                  #MenuBarIcon_MyInfo_ExportDatabase)
      
      AddEntry(#ShortcutValue_ImportDatabase,           LongTooltip\ImportDatabase,                  #MenuBarIcon_MyInfo_ImportDatabase)
      
      AddEntry(#ShortcutValue_OpenAdatabase,            LongTooltip\OpenAdatabase,                   #MenuBarIcon_MyInfo_OpenAdatabase)
      
      AddEntry(#ShortcutValue_VacuumDatabase,           LongTooltip\VacuumDatabase,                  #MenuBarIcon_MyInfo_VacuumDatabase)
      
      AddEntry(#ShortcutValue_ExitProgram,              LongTooltip\ExitProgram,                     #MenuBarIcon_MyInfo_ExitProgram)
      
      AddHeading(#Empty$, "Category menu, main window")
      
      AddEntry(#ShortcutValue_AddCategory,              LongTooltip\AddCategory,                     #MenuBarIcon_MyInfo_AddCategory)
      
      AddEntry(#ShortcutValue_CopyCategory,             LongTooltip\CopyCategory,                    #MenuBarIcon_MyInfo_CopyCategory)
      
      AddEntry(#ShortcutValue_DeleteCategory,           LongTooltip\DeleteCategory,                  #MenuBarIcon_MyInfo_DeleteCategory)
      
      AddEntry(#ShortcutValue_EditCategory,             LongTooltip\EditCategory,                    #MenuBarIcon_MyInfo_EditCategory)
      
      AddEntry(#ShortcutValue_ExportCategories,         LongTooltip\ExportCategories,                #MenuBarIcon_MyInfo_ExportCategories)
      
      AddEntry(#ShortcutValue_ImportCategories,         LongTooltip\ImportCategories,                #MenuBarIcon_MyInfo_ImportCategories)
      
      AddHeading(#Empty$, "Data menu, main window")
      
      AddEntry(#ShortcutValue_AddItem,                  LongTooltip\AddItem,                         #MenuBarIcon_MyInfo_AddItem)
      
      AddEntry(#ShortcutValue_ArchiveItem,              LongTooltip\ArchiveItem,                     #MenuBarIcon_MyInfo_ArchiveItem)
      
      AddEntry(#ShortcutValue_AttachItem,               LongTooltip\AttachItem,                      #MenuBarIcon_MyInfo_AttachItem)
      
      AddEntry(#ShortcutValue_CopyItem,                 LongTooltip\CopyItem,                        #MenuBarIcon_MyInfo_CopyItem)
      
      AddEntry(#ShortcutValue_DeleteItem,               LongTooltip\DeleteItem,                      #MenuBarIcon_MyInfo_DeleteItem)
      
      AddEntry(#ShortcutValue_EditItem,                 LongTooltip\EditItem,                        #MenuBarIcon_MyInfo_EditItem)
      
      AddEntry(#ShortcutValue_ExportItem,               LongTooltip\ExportItem,                      #MenuBarIcon_MyInfo_ExportItem)
      
      AddEntry(#ShortcutValue_FavouriteItem,            LongTooltip\FavouriteItem,                   #MenuBarIcon_MyInfo_FavouriteItem)
      
      AddEntry(#ShortcutValue_ImportItem,               LongTooltip\ImportItem,                      #MenuBarIcon_MyInfo_ImportItem)
      
      AddEntry(#ShortcutValue_KillItem,                 LongTooltip\KillItem,                        #MenuBarIcon_MyInfo_KillItem)
      
      AddEntry(#ShortcutValue_LockItem,                 LongTooltip\LockItem,                        #MenuBarIcon_MyInfo_LockItem)
      
      AddEntry(#ShortcutValue_PrintItem,                LongTooltip\PrintItem,                       #MenuBarIcon_MyInfo_PrintItem)
      
      AddHeading(#Empty$, "Utilities menu, main window")
      
      AddEntry(#ShortcutValue_AdvancedSearch,           LongTooltip\AdvancedSearch,                  #MenuBarIcon_MyInfo_AdvancedSearch)
      
      AddEntry(#ShortcutValue_SelectAllItems,           LongTooltip\SelectAllItems,                  #MenuBarIcon_MyInfo_SelectAllItems)
      
      AddEntry(#ShortcutValue_OwnerManagement,          LongTooltip\OwnerManagement,                 #MenuBarIcon_MyInfo_OwnerManagement)
      
      AddEntry(#ShortcutValue_ChangeFont,               LongTooltip\ChangeFont,                      #MenuBarIcon_MyInfo_ChangeFont)
      
      AddHeading(#Empty$, "Options menu, main window")
      
      AddEntry(#ShortcutValue_BalloonTooltips,          LongTooltip\BalloonTooltips,                 #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_BalloonColour,            LongTooltip\BalloonColour,                   #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_ColourBand,               LongTooltip\ColourBand,                      #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_HideDetail,               LongTooltip\HideDetail,                      #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_ShowToolbar,              LongTooltip\ShowToolbar,                     #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_ScreenSnap,               LongTooltip\ScreenSnap,                      #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_StickyWindow,             LongTooltip\StickyWindow,                    #Image_GenericBlank)
      
      AddEntry(#ShortcutValue_TitleClock,               LongTooltip\TitleClock,                      #Image_GenericBlank)
      
      AddHeading(#Empty$, "Help menu, main window")
      
      AddEntry(#ShortcutValue_AboutProgram,             LongTooltip\AboutProgram,                    #MenuBarIcon_MyInfo_AboutProgram)
      
      AddEntry(#ShortcutValue_KeyboardShortcuts,        LongTooltip\KeyboardShortcuts,               #MenuBarIcon_MyInfo_KeyboardShortcuts)
      
      AddEntry(#ShortcutValue_ProgramHelp,              LongTooltip\ProgramHelp,                     #MenuBarIcon_MyInfo_ProgramHelp)
      
      AddHeading(#Empty$, "Search function, main window")
      
      AddEntry(#ShortcutValue_CheckEnter,               LongTooltip\CheckEnter,                      #Image_Enter)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      AddHeading(#Empty$, "Category editor, category window")
      
      AddEntry(#ShortcutValue_SaveCategory,             LongTooltip\SaveCategory,                    #Image_Save)
      
      AddEntry(#ShortcutValue_HelpCategory,             LongTooltip\HelpCategory,                    #Image_Help)
      
      AddEntry(#ShortcutValue_ExitCategory,             LongTooltip\ExitCategory,                    #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeCategory,           LongTooltip\EscapeCategory,                  #Image_Escape)
      
      AddHeading(#Empty$, "Data editor, data window")
      
      AddEntry(#ShortcutValue_SaveData,                 LongTooltip\SaveData,                        #Image_Save)
      
      AddEntry(#ShortcutValue_ExportData,               LongTooltip\ExportData,                      #Image_Export)
      
      AddEntry(#ShortcutValue_ImportData,               LongTooltip\ImportData,                      #Image_Import)
      
     ;AddEntry(#ShortcutValue_SearchData,               LongTooltip\SearchData,                      #Image_Search)
      
      AddEntry(#ShortcutValue_HelpData,                 LongTooltip\HelpData,                        #Image_Help)
      
      AddEntry(#ShortcutValue_ExitData,                 LongTooltip\ExitData,                        #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeData,               LongTooltip\EscapeData,                      #Image_Escape)
      
      AddHeading(#Empty$, "Find text functions, Find text window")
      
      AddEntry(#ShortcutValue_FindFirst,                LongTooltip\FindFirst,                       #Image_First)
      
      AddEntry(#ShortcutValue_FindNext,                 LongTooltip\FindNext,                        #Image_Next)
      
      AddHeading(#Empty$, "Advanced search functions, Advanced search window")
      
      AddEntry(#ShortcutValue_RunSearch,                LongTooltip\RunSearch,                       #Image_Run)
      
      AddEntry(#ShortcutValue_HelpSearch,               LongTooltip\HelpSearch,                      #Image_Help)
      
      AddEntry(#ShortcutValue_ExitSearch,               LongTooltip\ExitSearch,                      #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeSearch,             LongTooltip\EscapeSearch,                    #Image_Escape)
      
      AddEntry(#ShortcutValue_ReturnSearch,             LongTooltip\ReturnSearch,                    #Image_Run)
      
      AddHeading(#Empty$, "Owner list functions, Owner list window")
      
      AddEntry(#ShortcutValue_SaveOwnerlist,            LongTooltip\SaveOwnerlist,                   #Image_Save)
      
      AddEntry(#ShortcutValue_AddOwnerlist,             LongTooltip\AddOwnerlist,                    #Image_Add)
      
      AddEntry(#ShortcutValue_DeleteOwnerlist,          LongTooltip\DeleteOwnerlist,                 #Image_Delete)
      
      AddEntry(#ShortcutValue_EditOwnerlist,            LongTooltip\EditOwnerlist,                   #Image_Edit)
      
      AddEntry(#ShortcutValue_HelpOwnerlist,            LongTooltip\HelpOwnerlist,                   #Image_Help)
      
      AddEntry(#ShortcutValue_Exitownerlist,            LongTooltip\Exitownerlist,                   #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeOwnerlist,          LongTooltip\EscapeOwnerlist,                 #Image_Escape)
      
      AddHeading(#Empty$, "Owner editor functions, Owner editor window")
      
      AddEntry(#ShortcutValue_SaveOwner,                LongTooltip\SaveOwner,                       #Image_Save)
      
      AddEntry(#ShortcutValue_HelpOwner,                LongTooltip\HelpOwner,                       #Image_Help)
      
      AddEntry(#ShortcutValue_ExitOwner,                LongTooltip\ExitOwner,                       #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeOwner,              LongTooltip\EscapeOwner,                     #Image_Escape)
      
      AddHeading(#Empty$, "Data print functions, Data print window")
      
      AddEntry(#ShortcutValue_NowPrint,                 LongTooltip\NowPrint,                        #Image_Print)
      
      AddEntry(#ShortcutValue_NowPreview,               LongTooltip\NowPreview,                      #Image_Preview)
      
      AddEntry(#ShortcutValue_ExitPrint,                LongTooltip\ExitPrint,                       #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapePrint,              LongTooltip\EscapePrint,                     #Image_Escape)
      
      AddHeading(#Empty$, "About program functions, About program window")
      
      AddEntry(#ShortcutValue_EscapeAbout,              LongTooltip\EscapeAbout,                     #Image_Escape)
      
      AddHeading(#Empty$, "Help display functions, Help display window")
      
      AddEntry(#ShortcutValue_LoadHelp,                 LongTooltip\LoadHelp,                        #Image_Load)
      
      AddEntry(#ShortcutValue_SaveHelp,                 LongTooltip\SaveHelp,                        #Image_Save)
      
      AddEntry(#ShortcutValue_SearchHelp,               LongTooltip\SearchHelp,                      #Image_Search)
      
      AddEntry(#ShortcutValue_PrintHelp,                LongTooltip\PrintHelp,                       #Image_Print)
      
      AddEntry(#ShortcutValue_EditHelp,                 LongTooltip\EditHelp,                        #Image_Edit)
      
      AddEntry(#ShortcutValue_ExitHelp,                 LongTooltip\ExitHelp,                        #Image_Exit)
      
      AddEntry(#ShortcutValue_EscapeHelp,               LongTooltip\EscapeHelp,                      #Image_Escape)
      
      AddEntry(#ShortcutValue_SearchOnEnter,            LongTooltip\SearchOnEnter,                   #Image_Enter)
      
      AddHeading(#Empty$, "Keyboard shortcuts functions, Keyboard shortcuts window")
      
      AddEntry(#ShortcutValue_EscapeShortcuts,          LongTooltip\EscapeShortcuts,                 #Image_Escape)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the current operation details on the the main window status bar
      ;--------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings",  "Info",  Message.s, #PB_Compiler_Procedure)
      
      ;--------------------------------------------------------------------------------------------
      ; Set focus on the appointment name field
      ;--------------------------------------------------------------------------------------------
      
      SetActiveGadget(#Gadget_Shortcuts_Keylist)
      
      ;--------------------------------------------------------------------------------------------
      ; Display status to the user
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Error",  "Could not open the [" + Program\LastWindow + "] window.", #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; Display status to the user
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  "The [" +  Program\LastWindow + "] window is still open, close that one first.", #PB_Compiler_Procedure)
    
  EndIf
  
  ;----------------------------------------------------------------------------------------------
  ; 
  ;----------------------------------------------------------------------------------------------
  
EndProcedure



; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 176
; FirstLine = 172
; Folding = -
; EnableXP
; HideErrorLog
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant